@@ -1,7 +1,7 @@
 /**************************************************************************
  *   nano.h  --  This file is part of GNU nano.                           *
  *                                                                        *
- *   Copyright (C) 1999-2011, 2013-2018 Free Software Foundation, Inc.    *
+ *   Copyright (C) 1999-2011, 2013-2019 Free Software Foundation, Inc.    *
  *   Copyright (C) 2014-2017 Benno Schulenberg                            *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
@@ -270,32 +270,32 @@ typedef struct lintstruct {
 #endif /* ENABLE_COLOR */
 
 /* More structure types. */
-typedef struct filestruct {
+typedef struct linestruct {
 	char *data;
 		/* The text of this line. */
 	ssize_t lineno;
 		/* The number of this line. */
-	struct filestruct *next;
+	struct linestruct *next;
 		/* Next node. */
-	struct filestruct *prev;
+	struct linestruct *prev;
 		/* Previous node. */
 #ifdef ENABLE_COLOR
 	short *multidata;
 		/* Array of which multi-line regexes apply to this line. */
 #endif
-} filestruct;
+} linestruct;
 
 typedef struct partition {
-	filestruct *fileage;
+	linestruct *filetop;
 		/* The top line of this portion of the file. */
-	filestruct *top_prev;
+	linestruct *top_prev;
 		/* The line before the top line of this portion of the file. */
 	char *top_data;
 		/* The text before the beginning of the top line of this portion
 		 * of the file. */
-	filestruct *filebot;
+	linestruct *filebot;
 		/* The bottom line of this portion of the file. */
-	filestruct *bot_next;
+	linestruct *bot_next;
 		/* The line after the bottom line of this portion of the
 		 * file. */
 	char *bot_data;
@@ -332,9 +332,9 @@ typedef struct undo {
 		/* Undo info specific to groups of lines. */
 
 	/* Cut-specific stuff we need. */
-	filestruct *cutbuffer;
+	linestruct *cutbuffer;
 		/* Copy of the cutbuffer. */
-	filestruct *cutbottom;
+	linestruct *cutbottom;
 		/* Copy of cutbottom. */
 	ssize_t mark_begin_lineno;
 		/* Mostly the line number of the current line; sometimes something else. */
@@ -360,13 +360,13 @@ typedef struct poshiststruct {
 typedef struct openfilestruct {
 	char *filename;
 		/* The file's name. */
-	filestruct *fileage;
+	linestruct *filetop;
 		/* The file's first line. */
-	filestruct *filebot;
+	linestruct *filebot;
 		/* The file's last line. */
-	filestruct *edittop;
+	linestruct *edittop;
 		/* The current top of the edit window for this file. */
-	filestruct *current;
+	linestruct *current;
 		/* The current line for this file. */
 	size_t totsize;
 		/* The file's total number of characters. */
@@ -384,7 +384,7 @@ typedef struct openfilestruct {
 	struct stat *current_stat;
 		/* The file's current stat information. */
 #ifndef NANO_TINY
-	filestruct *mark;
+	linestruct *mark;
 		/* The line in the file where the mark is set; NULL if not set. */
 	size_t mark_x;
 		/* The mark's x position in the above line. */
@@ -426,17 +426,17 @@ typedef struct rcoption {
 } rcoption;
 #endif
 
-typedef struct sc {
+typedef struct keystruct {
 	const char *keystr;
-		/* The string that describes a keystroke, like "^C" or "M-R". */
+		/* The string that describes the keystroke, like "^C" or "M-R". */
 	bool meta;
 		/* Whether this is a Meta keystroke. */
 	int keycode;
 		/* The integer that, together with meta, identifies the keystroke. */
 	int menus;
-		/* Which menus this applies to. */
+		/* The menus in which this keystroke is bound. */
 	void (*func)(void);
-		/* The function we're going to run. */
+		/* The function to which this keystroke is bound. */
 #ifndef NANO_TINY
 	int toggle;
 		/* If a toggle, what we're toggling. */
@@ -448,11 +448,11 @@ typedef struct sc {
 	char *expansion;
 		/* The string of keycodes to which this shortcut is expanded. */
 #endif
-	struct sc *next;
+	struct keystruct *next;
 		/* Next in the list. */
-} sc;
+} keystruct;
 
-typedef struct subnfunc {
+typedef struct funcstruct {
 	void (*func)(void);
 		/* The actual function to call. */
 	int menus;
@@ -470,9 +470,9 @@ typedef struct subnfunc {
 		/* Is this function allowed when in view mode? */
 	long toggle;
 		/* If this is a toggle, which toggle to affect. */
-	struct subnfunc *next;
+	struct funcstruct *next;
 		/* Next item in the list. */
-} subnfunc;
+} funcstruct;
 
 #ifdef ENABLE_WORDCOMPLETION
 typedef struct completion_word {
@@ -486,6 +486,7 @@ enum
 {
 	TITLE_BAR = 0,
 	LINE_NUMBER,
+	GUIDE_STRIPE,
 	SELECTED_TEXT,
 	STATUS_BAR,
 	ERROR_MESSAGE,
@@ -513,7 +514,7 @@ enum
 	MULTIBUFFER,
 	SMOOTH_SCROLL,
 	REBIND_DELETE,
-	REBIND_KEYPAD,
+	RAW_SEQUENCES,
 	NO_CONVERT,
 	BACKUP_FILE,
 	INSECURE_BACKUP,
@@ -529,7 +530,6 @@ enum
 	WORD_BOUNDS,
 	NO_NEWLINES,
 	BOLD_TEXT,
-	QUIET,
 	SOFTWRAP,
 	POSITIONLOG,
 	LOCKING,
@@ -541,7 +541,10 @@ enum
 	NO_PAUSES,
 	AT_BLANKS,
 	AFTER_ENDS,
-	LET_THEM_ZAP
+	LET_THEM_ZAP,
+	BREAK_LONG_LINES,
+	JUMPY_SCROLLING,
+	EMPTY_LINE
 };
 
 /* Flags for the menus in which a given function should be present. */
@@ -571,6 +574,7 @@ enum
 #endif
 
 /* Basic control codes. */
+#define BS_CODE   0x08
 #define TAB_CODE  0x09
 #define ESC_CODE  0x1B
 #define DEL_CODE  0x7F
